# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest
from google.api_core.gapic_v1.method import DEFAULT
from google.cloud.dataplex_v1.services.catalog_service.pagers import (
    ListAspectTypesPager,
    ListEntriesPager,
    ListEntryGroupsPager,
    ListEntryTypesPager,
    SearchEntriesPager,
)
from google.cloud.dataplex_v1.types import (
    ListAspectTypesRequest,
    ListAspectTypesResponse,
    ListEntriesRequest,
    ListEntriesResponse,
    ListEntryGroupsRequest,
    ListEntryGroupsResponse,
    ListEntryTypesRequest,
    ListEntryTypesResponse,
    SearchEntriesRequest,
    SearchEntriesResponse,
)

from airflow.exceptions import TaskDeferred
from airflow.providers.google.cloud.operators.dataplex import (
    DataplexCatalogCreateAspectTypeOperator,
    DataplexCatalogCreateEntryGroupOperator,
    DataplexCatalogCreateEntryOperator,
    DataplexCatalogCreateEntryTypeOperator,
    DataplexCatalogDeleteAspectTypeOperator,
    DataplexCatalogDeleteEntryGroupOperator,
    DataplexCatalogDeleteEntryOperator,
    DataplexCatalogDeleteEntryTypeOperator,
    DataplexCatalogGetAspectTypeOperator,
    DataplexCatalogGetEntryGroupOperator,
    DataplexCatalogGetEntryOperator,
    DataplexCatalogGetEntryTypeOperator,
    DataplexCatalogListAspectTypesOperator,
    DataplexCatalogListEntriesOperator,
    DataplexCatalogListEntryGroupsOperator,
    DataplexCatalogListEntryTypesOperator,
    DataplexCatalogLookupEntryOperator,
    DataplexCatalogSearchEntriesOperator,
    DataplexCreateAssetOperator,
    DataplexCreateLakeOperator,
    DataplexCreateOrUpdateDataProfileScanOperator,
    DataplexCreateOrUpdateDataQualityScanOperator,
    DataplexCreateTaskOperator,
    DataplexCreateZoneOperator,
    DataplexDeleteAssetOperator,
    DataplexDeleteDataProfileScanOperator,
    DataplexDeleteDataQualityScanOperator,
    DataplexDeleteLakeOperator,
    DataplexDeleteTaskOperator,
    DataplexDeleteZoneOperator,
    DataplexGetDataProfileScanResultOperator,
    DataplexGetDataQualityScanResultOperator,
    DataplexGetTaskOperator,
    DataplexListTasksOperator,
    DataplexRunDataProfileScanOperator,
    DataplexRunDataQualityScanOperator,
)
from airflow.providers.google.cloud.triggers.dataplex import DataplexDataQualityJobTrigger
from airflow.providers.google.common.consts import GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME

HOOK_STR = "airflow.providers.google.cloud.operators.dataplex.DataplexHook"
TASK_STR = "airflow.providers.google.cloud.operators.dataplex.Task"
LAKE_STR = "airflow.providers.google.cloud.operators.dataplex.Lake"
DATASCANJOB_STR = "airflow.providers.google.cloud.operators.dataplex.DataScanJob"
ZONE_STR = "airflow.providers.google.cloud.operators.dataplex.Zone"
ASSET_STR = "airflow.providers.google.cloud.operators.dataplex.Asset"
ENTRY_STR = "airflow.providers.google.cloud.operators.dataplex.Entry"
ENTRY_GROUP_STR = "airflow.providers.google.cloud.operators.dataplex.EntryGroup"
ENTRY_TYPE_STR = "airflow.providers.google.cloud.operators.dataplex.EntryType"
ASPECT_TYPE_STR = "airflow.providers.google.cloud.operators.dataplex.AspectType"

PROJECT_ID = "project-id"
REGION = "region"
LAKE_ID = "lake-id"
BODY = {"body": "test"}

BODY_LAKE = {
    "display_name": "test_display_name",
    "labels": [],
    "description": "test_description",
    "metastore": {"service": ""},
}
DATAPLEX_TASK_ID = "testTask001"

GCP_CONN_ID = "google_cloud_default"
API_VERSION = "v1"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]
DATA_SCAN_ID = "test-data-scan-id"
TASK_ID = "test_task_id"
ASSET_ID = "test_asset_id"
ZONE_ID = "test_zone_id"
JOB_ID = "test_job_id"
ENTRY_NAME = "test_entry"
ENTRY_GROUP_NAME = "test_entry_group"
ENTRY_TYPE_NAME = "test_entry_type"
ASPECT_TYPE_NAME = "test_aspect_type"
ENTRY_BODY = {
    "name": f"projects/{PROJECT_ID}/locations/{REGION}/entryGroups/{ENTRY_GROUP_NAME}/entries/{ENTRY_NAME}",
    "entry_type": f"projects/{PROJECT_ID}/locations/{REGION}/entryTypes/{ENTRY_TYPE_NAME}",
}


class TestDataplexCreateTaskOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(TASK_STR)
    def test_execute(self, task_mock, hook_mock):
        op = DataplexCreateTaskOperator(
            task_id="create_dataplex_task",
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            body=BODY,
            dataplex_task_id=DATAPLEX_TASK_ID,
            validate_only=None,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        task_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_task.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            body=BODY,
            dataplex_task_id=DATAPLEX_TASK_ID,
            validate_only=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteTaskOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteTaskOperator(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            dataplex_task_id=DATAPLEX_TASK_ID,
            task_id="delete_dataplex_task",
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=None)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_task.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            dataplex_task_id=DATAPLEX_TASK_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexListTasksOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexListTasksOperator(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            task_id="list_dataplex_task",
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.list_tasks.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            page_size=None,
            page_token=None,
            filter=None,
            order_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexGetTaskOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(TASK_STR)
    def test_execute(self, task_mock, hook_mock):
        op = DataplexGetTaskOperator(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            dataplex_task_id=DATAPLEX_TASK_ID,
            task_id="get_dataplex_task",
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        task_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_task.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            dataplex_task_id=DATAPLEX_TASK_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteLakeOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteLakeOperator(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            task_id="delete_dataplex_lake",
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_lake.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCreateLakeOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(LAKE_STR)
    def test_execute(self, lake_mock, hook_mock):
        op = DataplexCreateLakeOperator(
            task_id="create_dataplex_lake",
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            body=BODY_LAKE,
            validate_only=None,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        lake_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_lake.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            body=BODY_LAKE,
            validate_only=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexRunDataQualityScanOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute(self, mock_data_scan_job, hook_mock):
        op = DataplexRunDataQualityScanOperator(
            task_id="execute_data_scan",
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.run_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute_deferrable(self, mock_data_scan_job, hook_mock):
        op = DataplexRunDataQualityScanOperator(
            task_id="execute_data_scan",
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            deferrable=True,
        )

        with pytest.raises(TaskDeferred) as exc:
            op.execute(mock.MagicMock())

        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.run_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )
        hook_mock.return_value.wait_for_data_scan_job.assert_not_called()

        assert isinstance(exc.value.trigger, DataplexDataQualityJobTrigger)
        assert exc.value.method_name == GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME


class TestDataplexRunDataProfileScanOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute(self, mock_data_scan_job, hook_mock):
        op = DataplexRunDataProfileScanOperator(
            task_id="execute_data_scan",
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.run_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexGetDataQualityScanResultOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute(self, mock_data_scan_job, hook_mock):
        op = DataplexGetDataQualityScanResultOperator(
            task_id="get_data_scan_result",
            project_id=PROJECT_ID,
            region=REGION,
            job_id=JOB_ID,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            wait_for_results=False,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_data_scan_job.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            job_id=JOB_ID,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute_deferrable(self, mock_data_scan_job, hook_mock):
        op = DataplexGetDataQualityScanResultOperator(
            task_id="get_data_scan_result",
            project_id=PROJECT_ID,
            region=REGION,
            job_id=JOB_ID,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            wait_for_results=True,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            deferrable=True,
        )

        with pytest.raises(TaskDeferred) as exc:
            op.execute(mock.MagicMock())

        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.wait_for_data_scan_job.assert_not_called()

        assert isinstance(exc.value.trigger, DataplexDataQualityJobTrigger)
        assert exc.value.method_name == GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME


class TestDataplexGetDataProfileScanResultOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(DATASCANJOB_STR)
    def test_execute(self, mock_data_scan_job, hook_mock):
        op = DataplexGetDataProfileScanResultOperator(
            task_id="get_data_scan_result",
            project_id=PROJECT_ID,
            region=REGION,
            job_id=JOB_ID,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            wait_for_results=False,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_data_scan_job.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            job_id=JOB_ID,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCreateAssetOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(ASSET_STR)
    def test_execute(self, asset_mock, hook_mock):
        op = DataplexCreateAssetOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            asset_id=ASSET_ID,
            body={},
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        asset_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_asset.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            asset_id=ASSET_ID,
            body={},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCreateZoneOperator:
    @mock.patch(HOOK_STR)
    @mock.patch(ZONE_STR)
    def test_execute(self, zone_mock, hook_mock):
        op = DataplexCreateZoneOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            body={},
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        zone_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_zone.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            body={},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteZoneOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteZoneOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_zone.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteAssetOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteAssetOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            asset_id=ASSET_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_asset.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            lake_id=LAKE_ID,
            zone_id=ZONE_ID,
            asset_id=ASSET_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteDataQualityScanOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteDataQualityScanOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexDeleteDataProfileScanOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexDeleteDataProfileScanOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCreateDataQualityScanOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexCreateOrUpdateDataQualityScanOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        update_operator = DataplexCreateOrUpdateDataQualityScanOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            update_mask={},
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        update_operator.execute(context=mock.MagicMock())
        hook_mock.return_value.create_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )
        hook_mock.return_value.update_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            update_mask={},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCreateDataProfileScanOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexCreateOrUpdateDataProfileScanOperator(
            task_id=TASK_ID,
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_data_scan.assert_called_once_with(
            project_id=PROJECT_ID,
            region=REGION,
            data_scan_id=DATA_SCAN_ID,
            body={},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogCreateEntryGroupOperator:
    @mock.patch(ENTRY_GROUP_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_group_mock):
        op = DataplexCatalogCreateEntryGroupOperator(
            task_id="create_task",
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            entry_group_configuration=BODY,
            validate_request=None,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        entry_group_mock.return_value.to_dict.return_value = None
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_entry_group.assert_called_once_with(
            entry_group_id=ENTRY_GROUP_NAME,
            entry_group_configuration=BODY,
            location=REGION,
            project_id=PROJECT_ID,
            validate_only=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogGetEntryGroupOperator:
    @mock.patch(ENTRY_GROUP_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_group_mock):
        op = DataplexCatalogGetEntryGroupOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            task_id="get_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        entry_group_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_entry_group.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogDeleteEntryGroupOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexCatalogDeleteEntryGroupOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            task_id="delete_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_entry_group.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogListEntryGroupsOperator:
    @mock.patch(ENTRY_GROUP_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_group_mock):
        op = DataplexCatalogListEntryGroupsOperator(
            project_id=PROJECT_ID,
            location=REGION,
            task_id="list_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.list_entry_groups.return_value = ListEntryGroupsPager(
            response=(
                ListEntryGroupsResponse(
                    entry_groups=[
                        {
                            "name": "aaa",
                            "description": "Test Entry Group 1",
                            "display_name": "Entry Group One",
                        }
                    ]
                )
            ),
            method=mock.MagicMock(),
            request=ListEntryGroupsRequest(parent=""),
        )

        entry_group_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.list_entry_groups.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            page_size=None,
            page_token=None,
            filter_by=None,
            order_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogCreateEntryTypeOperator:
    @mock.patch(ENTRY_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_group_mock):
        op = DataplexCatalogCreateEntryTypeOperator(
            task_id="create_task",
            project_id=PROJECT_ID,
            location=REGION,
            entry_type_id=ENTRY_TYPE_NAME,
            entry_type_configuration=BODY,
            validate_request=None,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        entry_group_mock.return_value.to_dict.return_value = None
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_entry_type.assert_called_once_with(
            entry_type_id=ENTRY_TYPE_NAME,
            entry_type_configuration=BODY,
            location=REGION,
            project_id=PROJECT_ID,
            validate_only=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogGetEntryTypeOperator:
    @mock.patch(ENTRY_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_type_mock):
        op = DataplexCatalogGetEntryTypeOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_type_id=ENTRY_TYPE_NAME,
            task_id="get_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        entry_type_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_entry_type.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_type_id=ENTRY_TYPE_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogDeleteEntryTypeOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexCatalogDeleteEntryTypeOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_type_id=ENTRY_TYPE_NAME,
            task_id="delete_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_entry_type.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_type_id=ENTRY_TYPE_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogListEntryTypesOperator:
    @mock.patch(ENTRY_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_type_mock):
        op = DataplexCatalogListEntryTypesOperator(
            project_id=PROJECT_ID,
            location=REGION,
            task_id="list_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.list_entry_types.return_value = ListEntryTypesPager(
            response=(
                ListEntryTypesResponse(
                    entry_types=[
                        {
                            "name": "aaa",
                            "description": "Test Entry Type 1",
                            "display_name": "Entry Type One",
                        }
                    ]
                )
            ),
            method=mock.MagicMock(),
            request=ListEntryTypesRequest(parent=""),
        )

        entry_type_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.list_entry_types.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            page_size=None,
            page_token=None,
            filter_by=None,
            order_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogCreateAspectTypeOperator:
    @mock.patch(ASPECT_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, aspect_type_mock):
        op = DataplexCatalogCreateAspectTypeOperator(
            task_id="create_task",
            project_id=PROJECT_ID,
            location=REGION,
            aspect_type_id=ASPECT_TYPE_NAME,
            aspect_type_configuration=BODY,
            validate_request=None,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        aspect_type_mock.return_value.to_dict.return_value = None
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_aspect_type.assert_called_once_with(
            aspect_type_id=ASPECT_TYPE_NAME,
            aspect_type_configuration=BODY,
            location=REGION,
            project_id=PROJECT_ID,
            validate_only=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogGetAspectTypeOperator:
    @mock.patch(ASPECT_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, aspect_type_mock):
        op = DataplexCatalogGetAspectTypeOperator(
            project_id=PROJECT_ID,
            location=REGION,
            aspect_type_id=ASPECT_TYPE_NAME,
            task_id="get_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        aspect_type_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_aspect_type.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            aspect_type_id=ASPECT_TYPE_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogDeleteAspectTypeOperator:
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock):
        op = DataplexCatalogDeleteAspectTypeOperator(
            project_id=PROJECT_ID,
            location=REGION,
            aspect_type_id=ASPECT_TYPE_NAME,
            task_id="delete_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.wait_for_operation.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_aspect_type.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            aspect_type_id=ASPECT_TYPE_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogListAspectTypesOperator:
    @mock.patch(ASPECT_TYPE_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, aspect_type_mock):
        op = DataplexCatalogListAspectTypesOperator(
            project_id=PROJECT_ID,
            location=REGION,
            task_id="list_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.list_aspect_types.return_value = ListAspectTypesPager(
            response=(
                ListAspectTypesResponse(
                    aspect_types=[
                        {
                            "name": "aaa",
                            "description": "Test Aspect Type 1",
                            "display_name": "Aspect Type One",
                        }
                    ]
                )
            ),
            method=mock.MagicMock(),
            request=ListAspectTypesRequest(parent=""),
        )

        aspect_type_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.list_aspect_types.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            page_size=None,
            page_token=None,
            filter_by=None,
            order_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogCreateEntryOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogCreateEntryOperator(
            task_id="create_task",
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            entry_configuration=ENTRY_BODY,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        entry_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.create_entry.assert_called_once_with(
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            entry_configuration=ENTRY_BODY,
            location=REGION,
            project_id=PROJECT_ID,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogGetEntryOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogGetEntryOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            task_id="get_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        entry_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.get_entry.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            view=None,
            aspect_types=None,
            paths=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogLookupEntryOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogLookupEntryOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            task_id="lookup_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        entry_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.lookup_entry.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            view=None,
            aspect_types=None,
            paths=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogDeleteEntryOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogDeleteEntryOperator(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            task_id="delete_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        op.execute(context=mock.MagicMock())
        entry_mock.return_value.to_dict.return_value = None
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.delete_entry.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_id=ENTRY_NAME,
            entry_group_id=ENTRY_GROUP_NAME,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogListEntriesOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogListEntriesOperator(
            project_id=PROJECT_ID,
            location=REGION,
            task_id="list_task",
            entry_group_id=ENTRY_GROUP_NAME,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.list_entries.return_value = ListEntriesPager(
            response=(
                ListEntriesResponse(
                    entries=[
                        {
                            "name": f"projects/{PROJECT_ID}/locations/{REGION}/entryGroups/{ENTRY_GROUP_NAME}/entries/{ENTRY_NAME}",
                            "entry_type": f"projects/{PROJECT_ID}/locations/{REGION}/entryTypes/{ENTRY_TYPE_NAME}",
                        }
                    ]
                )
            ),
            method=mock.MagicMock(),
            request=ListEntriesRequest(parent=""),
        )

        entry_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.list_entries.assert_called_once_with(
            project_id=PROJECT_ID,
            location=REGION,
            entry_group_id=ENTRY_GROUP_NAME,
            page_size=None,
            page_token=None,
            filter_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestDataplexCatalogSearchEntriesOperator:
    @mock.patch(ENTRY_STR)
    @mock.patch(HOOK_STR)
    def test_execute(self, hook_mock, entry_mock):
        op = DataplexCatalogSearchEntriesOperator(
            project_id=PROJECT_ID,
            location="global",
            task_id="search_task",
            query="displayname:Display Name",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        hook_mock.return_value.search_entries.return_value = SearchEntriesPager(
            response=(
                SearchEntriesResponse(
                    results=[
                        {
                            "dataplex_entry": {
                                "name": f"projects/{PROJECT_ID}/locations/{REGION}/entryGroups/{ENTRY_GROUP_NAME}/entries/{ENTRY_NAME}",
                                "entry_type": f"projects/{PROJECT_ID}/locations/{REGION}/entryTypes/{ENTRY_TYPE_NAME}",
                            }
                        }
                    ]
                )
            ),
            method=mock.MagicMock(),
            request=SearchEntriesRequest(name="", query=""),
        )

        entry_mock.return_value.to_dict.return_value = None
        op.execute(context=mock.MagicMock())
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        hook_mock.return_value.search_entries.assert_called_once_with(
            project_id=PROJECT_ID,
            location="global",
            query="displayname:Display Name",
            page_size=None,
            page_token=None,
            order_by=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

from unittest import mock

from google.cloud.aiplatform_v1beta1.types import (
    FeatureOnlineStore,
    FeatureView,
    FeatureViewDataFormat,
    FeatureViewDataKey,
    FetchFeatureValuesResponse,
)

from airflow.providers.google.cloud.operators.vertex_ai.feature_store import (
    CreateFeatureOnlineStoreOperator,
    CreateFeatureViewOperator,
    DeleteFeatureOnlineStoreOperator,
    DeleteFeatureViewOperator,
    FetchFeatureValuesOperator,
    GetFeatureOnlineStoreOperator,
    GetFeatureViewSyncOperator,
    SyncFeatureViewOperator,
)

VERTEX_AI_PATH = "airflow.providers.google.cloud.operators.vertex_ai.{}"

TASK_ID = "test_task_id"
GCP_PROJECT = "test-project"
GCP_LOCATION = "us-central1"
GCP_CONN_ID = "test-conn"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]
FEATURE_ONLINE_STORE_ID = "test-store"
FEATURE_VIEW_ID = "test-view"
FEATURE_ONLINE_STORE_NAME = (
    f"projects/{GCP_PROJECT}/locations/{GCP_LOCATION}/featureOnlineStores/{FEATURE_ONLINE_STORE_ID}"
)
FEATURE_VIEW_SYNC_NAME = (
    f"projects/{GCP_PROJECT}/locations/{GCP_LOCATION}/featureOnlineStores/"
    f"{FEATURE_ONLINE_STORE_ID}/featureViews/{FEATURE_VIEW_ID}/featureViewSyncs/sync123"
)
FEATURE_VIEW_NAME = (
    f"projects/{GCP_PROJECT}/locations/{GCP_LOCATION}/featureOnlineStores/"
    f"{FEATURE_ONLINE_STORE_ID}/featureViews/{FEATURE_VIEW_ID}"
)


class TestSyncFeatureViewOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        # Create the mock hook and set up its return value
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        # Set up the return value for sync_feature_view to match the hook implementation
        mock_hook.sync_feature_view.return_value = FEATURE_VIEW_SYNC_NAME
        op = SyncFeatureViewOperator(
            task_id=TASK_ID,
            project_id=GCP_PROJECT,
            location=GCP_LOCATION,
            feature_online_store_id=FEATURE_ONLINE_STORE_ID,
            feature_view_id=FEATURE_VIEW_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        response = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.sync_feature_view.assert_called_once_with(
            project_id=GCP_PROJECT,
            location=GCP_LOCATION,
            feature_online_store_id=FEATURE_ONLINE_STORE_ID,
            feature_view_id=FEATURE_VIEW_ID,
        )
        # Verify response matches expected value
        assert response == FEATURE_VIEW_SYNC_NAME


class TestGetFeatureViewSyncOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        # Create the mock hook and set up expected response
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        expected_response = {
            "name": FEATURE_VIEW_SYNC_NAME,
            "start_time": 1000,
            "end_time": 2000,
            "sync_summary": {"row_synced": 500, "total_slot": 4},
        }
        # Set up the return value for get_feature_view_sync to match the hook implementation
        mock_hook.get_feature_view_sync.return_value = expected_response
        op = GetFeatureViewSyncOperator(
            task_id=TASK_ID,
            location=GCP_LOCATION,
            feature_view_sync_name=FEATURE_VIEW_SYNC_NAME,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        response = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.get_feature_view_sync.assert_called_once_with(
            location=GCP_LOCATION,
            feature_view_sync_name=FEATURE_VIEW_SYNC_NAME,
        )
        # Verify response matches expected structure
        assert response == expected_response


class TestCreateFeatureOnlineStoreOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        FEATURE_ONLINE_STORE_CONF = FeatureOnlineStore({"name": FEATURE_ONLINE_STORE_NAME, "optimized": {}})
        sample_result = {
            "etag": "",
            "labels": {},
            "name": FEATURE_ONLINE_STORE_NAME,
            "optimized": {},
            "satisfies_pzi": False,
            "satisfies_pzs": False,
            "state": 0,
        }
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        # Set up the return value for hook method to match the hook implementation
        sample_operation = mock.MagicMock()
        sample_operation.result.return_value = FEATURE_ONLINE_STORE_CONF
        mock_hook.create_feature_online_store.return_value = sample_operation
        mock_hook.return_value.wait_for_operation_result.side_effect = lambda operation: operation.result()
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "feature_online_store": FEATURE_ONLINE_STORE_CONF,
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = CreateFeatureOnlineStoreOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        result = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.create_feature_online_store.assert_called_once_with(**common_kwargs)
        # Verify result matches expected value
        assert result == sample_result


class TestCreateFeatureViewOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        feature_view_conf_params = {
            "big_query_source": FeatureView.BigQuerySource(
                uri="bq://{BQ_TABLE}",
                entity_id_columns=["entity_id"],
            ),
            "sync_config": FeatureView.SyncConfig(cron="TZ=Europe/London 56 * * * *"),
        }
        FEATURE_VIEW_CONF = FeatureView(**feature_view_conf_params)

        sample_result = {
            "big_query_source": {"entity_id_columns": ["entity_id"], "uri": "bq://{BQ_TABLE}"},
            "etag": "",
            "labels": {},
            "name": "projects/test-project/locations/us-central1/featureOnlineStores/test-store"
            "/featureViews/test-view",
            "satisfies_pzi": False,
            "satisfies_pzs": False,
            "service_account_email": "",
            "service_agent_type": 0,
            "sync_config": {"cron": "TZ=Europe/London 56 * * * *"},
        }
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        # Set up the return value for hook method to match the hook implementation
        sample_operation = mock.MagicMock()
        sample_operation.result.return_value = FeatureView(
            {**feature_view_conf_params, **{"name": FEATURE_VIEW_NAME}}
        )
        mock_hook.create_feature_view.return_value = sample_operation
        mock_hook.return_value.wait_for_operation_result.side_effect = lambda operation: operation.result()
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "feature_view_id": FEATURE_VIEW_ID,
            "feature_view": FEATURE_VIEW_CONF,
            "run_sync_immediately": False,
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = CreateFeatureViewOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        result = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.create_feature_view.assert_called_once_with(**common_kwargs)
        # Verify result matches expected value
        assert result == sample_result


class TestFetchFeatureValuesOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        ENTITY_ID = "entity-id"
        FEATURE_VIEW_DATA_KEY = {"key": "28098"}
        sample_result = {
            "key_values": {
                "features": [
                    {"name": "brand", "value": {"string_value": "rip curl"}},
                    {"name": "cost", "value": {"double_value": 36.56684834767282}},
                    {"name": "feature_timestamp", "value": {"int64_value": "1750151356612667"}},
                ]
            }
        }
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        # Set up the return value for hook method to match the hook implementation
        mock_hook.get_feature_online_store = mock.MagicMock()
        PUBLIC_DOMAIN_NAME = "public.domain.url"
        get_public_domain_mock = mock.MagicMock()
        get_public_domain_mock.return_value = PUBLIC_DOMAIN_NAME
        mock_hook._get_featurestore_public_endpoint = get_public_domain_mock
        mock_hook.fetch_feature_values.return_value = FetchFeatureValuesResponse(sample_result)
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "feature_view_id": FEATURE_VIEW_ID,
            "entity_id": ENTITY_ID,
            "data_key": FeatureViewDataKey(FEATURE_VIEW_DATA_KEY),
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = FetchFeatureValuesOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        result = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.fetch_feature_values.assert_called_once_with(
            data_format=FeatureViewDataFormat.KEY_VALUE,
            endpoint_domain_name=PUBLIC_DOMAIN_NAME,
            **common_kwargs,
        )
        # Verify result matches expected value
        assert result == sample_result


class TestGetFeatureOnlineStoreOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        # Create the mock hook and set up its return value
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        # Set up the return value for get_feature_online_store to match the hook implementation
        SAMPLE_RESPONSE = {
            "etag": "",
            "labels": {},
            "name": FEATURE_ONLINE_STORE_ID,
            "satisfies_pzi": False,
            "satisfies_pzs": False,
            "state": 0,
        }
        mock_hook.get_feature_online_store.return_value = FeatureOnlineStore(SAMPLE_RESPONSE)
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = GetFeatureOnlineStoreOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        response = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.get_feature_online_store.assert_called_once_with(**common_kwargs)
        # Verify response matches expected value
        assert response == SAMPLE_RESPONSE


class TestDeleteFeatureOnlineStoreOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        # Create the mock hook and set up its return value
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        sample_operation = mock.MagicMock()
        mock_hook.delete_feature_online_store.return_value = sample_operation
        mock_hook.return_value.wait_for_operation_result.side_effect = lambda operation: operation.result()
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "force": False,
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = DeleteFeatureOnlineStoreOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        response = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.delete_feature_online_store.assert_called_once_with(**common_kwargs)
        # Verify response matches expected value
        assert response == {"result": f"The {FEATURE_ONLINE_STORE_ID} has been deleted."}


class TestDeleteFeatureViewOperator:
    @mock.patch(VERTEX_AI_PATH.format("feature_store.FeatureStoreHook"))
    def test_execute(self, mock_hook_class):
        # Create the mock hook and set up its return value
        mock_hook = mock.MagicMock()
        mock_hook_class.return_value = mock_hook
        sample_operation = mock.MagicMock()
        mock_hook.delete_feature_view.return_value = sample_operation
        mock_hook.return_value.wait_for_operation_result.side_effect = lambda operation: operation.result()
        common_kwargs = {
            "project_id": GCP_PROJECT,
            "location": GCP_LOCATION,
            "feature_online_store_id": FEATURE_ONLINE_STORE_ID,
            "feature_view_id": FEATURE_VIEW_ID,
            "metadata": (),
            "timeout": 100,
            "retry": None,
        }
        op = DeleteFeatureViewOperator(
            task_id=TASK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            **common_kwargs,
        )
        response = op.execute(context={"ti": mock.MagicMock()})
        # Verify hook initialization
        mock_hook_class.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        # Verify hook method call
        mock_hook.delete_feature_view.assert_called_once_with(**common_kwargs)
        # Verify response matches expected value
        assert response == {"result": f"The {FEATURE_VIEW_ID} has been deleted."}



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from typing import TYPE_CHECKING

from google.cloud.container_v1.types import Cluster

from airflow.providers.google.cloud.links.base import BaseGoogleLink

if TYPE_CHECKING:
    from airflow.utils.context import Context

KUBERNETES_BASE_LINK = "/kubernetes"
KUBERNETES_CLUSTER_LINK = (
    KUBERNETES_BASE_LINK + "/clusters/details/{location}/{cluster_name}/details?project={project_id}"
)
KUBERNETES_POD_LINK = (
    KUBERNETES_BASE_LINK
    + "/pod/{location}/{cluster_name}/{namespace}/{pod_name}/details?project={project_id}"
)
KUBERNETES_JOB_LINK = (
    KUBERNETES_BASE_LINK
    + "/job/{location}/{cluster_name}/{namespace}/{job_name}/details?project={project_id}"
)
KUBERNETES_WORKLOADS_LINK = (
    KUBERNETES_BASE_LINK + '/workload/overview?project={project_id}&pageState=("savedViews":'
    '("c":%5B"gke%2F{location}%2F{cluster_name}"%5D,"n":%5B"{namespace}"%5D))'
)


class KubernetesEngineClusterLink(BaseGoogleLink):
    """Helper class for constructing Kubernetes Engine Cluster Link."""

    name = "Kubernetes Cluster"
    key = "kubernetes_cluster_conf"
    format_str = KUBERNETES_CLUSTER_LINK

    @classmethod
    def persist(cls, context: Context, **value):
        cluster = value.get("cluster")
        if isinstance(cluster, dict):
            cluster = Cluster.from_json(json.dumps(cluster))

        if not cluster:
            raise ValueError("Cluster must be provided for KubernetesEngineClusterLink.")

        super().persist(
            context=context,
            cluster_name=cluster.name,
        )


class KubernetesEnginePodLink(BaseGoogleLink):
    """Helper class for constructing Kubernetes Engine Pod Link."""

    name = "Kubernetes Pod"
    key = "kubernetes_pod_conf"
    format_str = KUBERNETES_POD_LINK


class KubernetesEngineJobLink(BaseGoogleLink):
    """Helper class for constructing Kubernetes Engine Job Link."""

    name = "Kubernetes Job"
    key = "kubernetes_job_conf"
    format_str = KUBERNETES_JOB_LINK


class KubernetesEngineWorkloadsLink(BaseGoogleLink):
    """Helper class for constructing Kubernetes Engine Workloads Link."""

    name = "Kubernetes Workloads"
    key = "kubernetes_workloads_conf"
    format_str = KUBERNETES_WORKLOADS_LINK



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections import OrderedDict
from unittest import mock
from unittest.mock import MagicMock, PropertyMock

import pytest
import sqlalchemy

from airflow.providers.google.cloud.hooks.spanner import SpannerHook
from airflow.providers.google.common.consts import CLIENT_INFO

from unit.google.cloud.utils.base_gcp_mock import (
    GCP_PROJECT_ID_HOOK_UNIT_TEST,
    mock_base_gcp_hook_default_project_id,
    mock_base_gcp_hook_no_default_project_id,
)

SPANNER_INSTANCE = "instance"
SPANNER_CONFIGURATION = "configuration"
SPANNER_DATABASE = "database-name"
SPANNER_PROJECT_ID = "test_project_id"
SPANNER_CONN_PARAMS = (SPANNER_PROJECT_ID, SPANNER_INSTANCE, SPANNER_DATABASE)


class TestGcpSpannerHookDefaultProjectId:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
            new=mock_base_gcp_hook_default_project_id,
        ):
            self.spanner_hook_default_project_id = SpannerHook(gcp_conn_id="test")

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook.get_credentials")
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.Client")
    def test_spanner_client_creation(self, mock_client, mock_get_creds):
        result = self.spanner_hook_default_project_id._get_client(GCP_PROJECT_ID_HOOK_UNIT_TEST)
        mock_client.assert_called_once_with(
            project=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            credentials=mock_get_creds.return_value,
            client_info=CLIENT_INFO,
        )
        assert mock_client.return_value == result
        assert self.spanner_hook_default_project_id._client == result

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_existing_instance(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.get_instance(
            instance_id=SPANNER_INSTANCE, project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        assert res is not None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_existing_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.get_instance(
            instance_id=SPANNER_INSTANCE, project_id="new-project"
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        assert res is not None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_instance(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        create_method = instance_method.return_value.create
        create_method.return_value = False
        res = self.spanner_hook_default_project_id.create_instance(
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=1,
            display_name=SPANNER_DATABASE,
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=1,
        )
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        create_method = instance_method.return_value.create
        create_method.return_value = False
        res = self.spanner_hook_default_project_id.create_instance(
            project_id="new-project",
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=1,
            display_name=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=1,
        )
        assert res is None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_instance(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        update_method = instance_method.return_value.update
        update_method.return_value = False
        res = self.spanner_hook_default_project_id.update_instance(
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=2,
            display_name=SPANNER_DATABASE,
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=2,
        )
        update_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        update_method = instance_method.return_value.update
        update_method.return_value = False
        res = self.spanner_hook_default_project_id.update_instance(
            project_id="new-project",
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=2,
            display_name=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=2,
        )
        update_method.assert_called_once_with()
        assert res is None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_instance(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        delete_method = instance_method.return_value.delete
        delete_method.return_value = False
        res = self.spanner_hook_default_project_id.delete_instance(
            instance_id=SPANNER_INSTANCE,
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with("instance")
        delete_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        delete_method = instance_method.return_value.delete
        delete_method.return_value = False
        res = self.spanner_hook_default_project_id.delete_instance(
            project_id="new-project", instance_id=SPANNER_INSTANCE
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with("instance")
        delete_method.assert_called_once_with()
        assert res is None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_database(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_exists_method = instance_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.get_database(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        assert res is not None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_exists_method = instance_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.get_database(
            project_id="new-project", instance_id=SPANNER_INSTANCE, database_id=SPANNER_DATABASE
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        assert res is not None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_database(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_create_method = database_method.return_value.create
        res = self.spanner_hook_default_project_id.create_database(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name", ddl_statements=[])
        database_create_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_create_method = database_method.return_value.create
        res = self.spanner_hook_default_project_id.create_database(
            project_id="new-project",
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name", ddl_statements=[])
        database_create_method.assert_called_once_with()
        assert res is None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_database(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_update_ddl_method = database_method.return_value.update_ddl
        res = self.spanner_hook_default_project_id.update_database(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_update_ddl_method.assert_called_once_with(ddl_statements=[], operation_id=None)
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_update_ddl_method = database_method.return_value.update_ddl
        res = self.spanner_hook_default_project_id.update_database(
            project_id="new-project",
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_update_ddl_method.assert_called_once_with(ddl_statements=[], operation_id=None)
        assert res is None

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_database(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_drop_method = database_method.return_value.drop
        database_exists_method = database_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.delete_database(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        database_drop_method.assert_called_once_with()
        assert res

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_drop_method = database_method.return_value.drop
        database_exists_method = database_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_default_project_id.delete_database(
            project_id="new-project", instance_id=SPANNER_INSTANCE, database_id=SPANNER_DATABASE
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        database_drop_method.assert_called_once_with()
        assert res

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.project_id",
        new_callable=PropertyMock,
        return_value=GCP_PROJECT_ID_HOOK_UNIT_TEST,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_execute_dml(self, get_client, mock_project_id):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        run_in_transaction_method = database_method.return_value.run_in_transaction
        res = self.spanner_hook_default_project_id.execute_dml(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            queries=[""],
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        run_in_transaction_method.assert_called_once_with(mock.ANY)
        assert res == []

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_execute_dml_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        run_in_transaction_method = database_method.return_value.run_in_transaction
        res = self.spanner_hook_default_project_id.execute_dml(
            project_id="new-project", instance_id=SPANNER_INSTANCE, database_id=SPANNER_DATABASE, queries=[""]
        )
        get_client.assert_called_once_with(project_id="new-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        run_in_transaction_method.assert_called_once_with(mock.ANY)
        assert res == []

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_execute_dml_oqueries_row_count(self, get_client):
        pass

    @pytest.mark.parametrize(
        "returned_items, expected_counts",
        [
            pytest.param(
                [
                    ("DELETE FROM T WHERE archived = TRUE", 5),
                    ("SELECT * FROM T", 42),
                    ("UPDATE U SET flag = FALSE WHERE x = 1", 3),
                ],
                [5, 3],
            ),
            pytest.param(
                [
                    ("DELETE FROM Logs WHERE created_at < '2024-01-01'", 7),
                ],
                [7],
            ),
            pytest.param(
                [
                    (
                        "UPDATE Accounts SET active=false WHERE last_login < DATE_SUB(CURRENT_DATE(), INTERVAL 365 DAY)",
                        11,
                    ),
                    ("DELETE FROM Sessions WHERE expires_at < CURRENT_TIMESTAMP()", 23),
                ],
                [11, 23],
            ),
            pytest.param(
                [
                    ("SELECT COUNT(*) FROM Users", 50000),
                    ("SELECT * FROM BigTable", 123456),
                ],
                [],
            ),
            pytest.param(
                [],
                [],
            ),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_execute_dml_parametrized(self, get_client, returned_items, expected_counts):
        instance_method = get_client.return_value.instance
        database_method = instance_method.return_value.database
        run_in_tx = database_method.return_value.run_in_transaction

        returned_mapping = OrderedDict(returned_items)
        run_in_tx.return_value = returned_mapping

        res = self.spanner_hook_default_project_id.execute_dml(
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            queries=[sql for sql, _ in returned_items],
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
        )

        assert res == expected_counts

    def test_get_uri(self):
        self.spanner_hook_default_project_id._get_conn_params = MagicMock(return_value=SPANNER_CONN_PARAMS)
        uri = self.spanner_hook_default_project_id.get_uri()
        assert (
            uri
            == f"spanner+spanner:///projects/{SPANNER_PROJECT_ID}/instances/{SPANNER_INSTANCE}/databases/{SPANNER_DATABASE}"
        )

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_sqlalchemy_engine(self, get_client):
        self.spanner_hook_default_project_id._get_conn_params = MagicMock(return_value=SPANNER_CONN_PARAMS)
        engine = self.spanner_hook_default_project_id.get_sqlalchemy_engine()
        assert isinstance(engine, sqlalchemy.engine.Engine)
        assert engine.name == "spanner+spanner"


class TestGcpSpannerHookNoDefaultProjectID:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
            new=mock_base_gcp_hook_no_default_project_id,
        ):
            self.spanner_hook_no_default_project_id = SpannerHook(gcp_conn_id="test")

    @mock.patch(
        "airflow.providers.google.cloud.hooks.spanner.SpannerHook.get_credentials",
        return_value="CREDENTIALS",
    )
    @mock.patch("airflow.providers.google.cloud.hooks.spanner.Client")
    def test_spanner_client_creation(self, mock_client, mock_get_creds):
        result = self.spanner_hook_no_default_project_id._get_client(GCP_PROJECT_ID_HOOK_UNIT_TEST)
        mock_client.assert_called_once_with(
            project=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            credentials=mock_get_creds.return_value,
            client_info=CLIENT_INFO,
        )
        assert mock_client.return_value == result
        assert self.spanner_hook_no_default_project_id._client == result

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_existing_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        res = self.spanner_hook_no_default_project_id.get_instance(
            instance_id=SPANNER_INSTANCE, project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        assert res is not None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_non_existing_instance(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = False
        res = self.spanner_hook_no_default_project_id.get_instance(
            instance_id=SPANNER_INSTANCE, project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        create_method = instance_method.return_value.create
        create_method.return_value = False
        res = self.spanner_hook_no_default_project_id.create_instance(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=1,
            display_name=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=1,
        )
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        update_method = instance_method.return_value.update
        update_method.return_value = False
        res = self.spanner_hook_no_default_project_id.update_instance(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            configuration_name=SPANNER_CONFIGURATION,
            node_count=2,
            display_name=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(
            instance_id="instance",
            configuration_name="configuration",
            display_name="database-name",
            node_count=2,
        )
        update_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_instance_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        delete_method = instance_method.return_value.delete
        delete_method.return_value = False
        res = self.spanner_hook_no_default_project_id.delete_instance(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST, instance_id=SPANNER_INSTANCE
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with("instance")
        delete_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_exists_method = instance_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_no_default_project_id.get_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        assert res is not None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_create_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_create_method = database_method.return_value.create
        res = self.spanner_hook_no_default_project_id.create_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name", ddl_statements=[])
        database_create_method.assert_called_once_with()
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_update_ddl_method = database_method.return_value.update_ddl
        res = self.spanner_hook_no_default_project_id.update_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            ddl_statements=[],
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_update_ddl_method.assert_called_once_with(ddl_statements=[], operation_id=None)
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_update_database_overridden_project_id_and_operation(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_update_ddl_method = database_method.return_value.update_ddl
        res = self.spanner_hook_no_default_project_id.update_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            operation_id="operation",
            ddl_statements=[],
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_update_ddl_method.assert_called_once_with(ddl_statements=[], operation_id="operation")
        assert res is None

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_database_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_drop_method = database_method.return_value.drop
        database_exists_method = database_method.return_value.exists
        database_exists_method.return_value = True
        res = self.spanner_hook_no_default_project_id.delete_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        database_drop_method.assert_called_once_with()
        assert res

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_delete_database_missing_database(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        database_drop_method = database_method.return_value.drop
        database_exists_method = database_method.return_value.exists
        database_exists_method.return_value = False
        self.spanner_hook_no_default_project_id.delete_database(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        database_exists_method.assert_called_once_with()
        database_drop_method.assert_not_called()

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_execute_dml_overridden_project_id(self, get_client):
        instance_method = get_client.return_value.instance
        instance_exists_method = instance_method.return_value.exists
        instance_exists_method.return_value = True
        database_method = instance_method.return_value.database
        run_in_transaction_method = database_method.return_value.run_in_transaction
        res = self.spanner_hook_no_default_project_id.execute_dml(
            project_id=GCP_PROJECT_ID_HOOK_UNIT_TEST,
            instance_id=SPANNER_INSTANCE,
            database_id=SPANNER_DATABASE,
            queries=[""],
        )
        get_client.assert_called_once_with(project_id="example-project")
        instance_method.assert_called_once_with(instance_id="instance")
        database_method.assert_called_once_with(database_id="database-name")
        run_in_transaction_method.assert_called_once_with(mock.ANY)
        assert res == []

    def test_get_uri(self):
        self.spanner_hook_no_default_project_id._get_conn_params = MagicMock(return_value=SPANNER_CONN_PARAMS)
        uri = self.spanner_hook_no_default_project_id.get_uri()
        assert (
            uri
            == f"spanner+spanner:///projects/{SPANNER_PROJECT_ID}/instances/{SPANNER_INSTANCE}/databases/{SPANNER_DATABASE}"
        )

    @mock.patch("airflow.providers.google.cloud.hooks.spanner.SpannerHook._get_client")
    def test_get_sqlalchemy_engine(self, get_client):
        self.spanner_hook_no_default_project_id._get_conn_params = MagicMock(return_value=SPANNER_CONN_PARAMS)
        engine = self.spanner_hook_no_default_project_id.get_sqlalchemy_engine()
        assert isinstance(engine, sqlalchemy.engine.Engine)
        assert engine.name == "spanner+spanner"
